#!/usr/bin/perl
#
use Statistics::OLS;
use Math::FFT;
use Math::Stat;
use Math::Spline qw(spline linsearch binsearch);
use Math::Derivative qw(Derivative2);
use Math::Approx;
use Astro::FITS::CFITSIO qw( :longnames :constants );
use PDL;
use PDL::Fit::Polynomial; 
use PDL::Filter::Linear;
use PGPLOT;  # Load PGPLOT module
use PDL::Fit::Gaussian;
use PDL::Slatec;
use PDL::Image2D;
#use PDL::Matrix;

#require "/work1/ssa/perl/MY/my.pl";
require "/home/sanchez/sda2/code/R3D/my.pl";

if ($#ARGV<1) {
    print "USE: Mosaic.pl CONFIG.txt OUTPUT\n";
    exit;
}

$config=$ARGV[0];
$outfile=$ARGV[1];
$nf=0;
$dxmin=1e12;
$dxmax=-1e12;
$dymin=1e12;
$dymax=-1e12;
open(FH,"<$config");
while($line=<FH>) {
#    chop($line);
    @data=split(" ",$line);
    $infile[$nf]=$data[0];
    $dx[$nf]=$data[1];
    $dy[$nf]=$data[2];
    if ($dxmin>$dx[$nf]) {
	$dxmin=$dx[$nf];
    }
    if ($dxmax<$dx[$nf]) {
	$dxmax=$dx[$nf];
    }
    if ($dymin>$dy[$nf]) {
	$dymin=$dy[$nf];
    }
    if ($dymax<$dy[$nf]) {
	$dymax=$dy[$nf];
    }
    $nf++;
}
close(FH);
print "$nf files\n";
$nax=read_naxes($infile[0]);   
@naxis=@$nax;
$nx=$naxis[0];
$ny=$naxis[1];
$nz=$naxis[2];
print "[$nx,$ny,$nz]\n";
@tr=read_img_headers($infile[0],["CRPIX1","CRVAL1","CDELT1","CRPIX2","CRVAL2","CDELT2","CRPIX3","CRVAL3","CDELT3"]);


$xmin=int($dxmin);
$xmax=int($nx+$dxmax);
$ymin=int($dymin);
$ymax=int($ny+$dymax);

$nx_new=$xmax-$xmin;
$ny_new=$ymax-$ymin;

print "$xmin $xmax $ymin $ymax $nx_new $ny_new\n";

for ($k=0;$k<$nz;$k++) {
    for ($j=0;$j<$ny_new;$j++) {
	for ($i=0;$i<$nx_new;$i++) {
	    $out_array[$k][$j][$i]=0;
	}
    }
}

for ($f=0;$f<$nf;$f++) {
    print "Reading file $infile[$f], ";    
    my @in_array=read_cube($infile[$f]);
    $nx_start=$dx[$f]-$dxmin;
    $ny_start=$dy[$f]-$dymin;
    print "added at $nx_start,$ny_start\n";
    for ($k=0;$k<$nz;$k++) {
	for ($j=0;$j<$ny;$j++) {
	    for ($i=0;$i<$nx;$i++) {
		$out_array[$k][$j+$ny_start][$i+$nx_start]+=$in_array[$k][$j][$i];
		$sum[$k][$j+$ny_start][$i+$nx_start]+=1;
	    }
	}
    }
}

for ($k=0;$k<$nz;$k++) {
    for ($j=0;$j<$ny_new;$j++) {
	for ($i=0;$i<$nx_new;$i++) {
	    if ($sum[$k][$j][$i]>0) {
		$out_array[$k][$j][$i]/=$sum[$k][$j][$i];
	    }
	}
    }
}

print "DONE\n";
print "Writting $outfile\n";
$nax=[$nx_new,$ny_new,$nz];
system("rm $outfile");
write_fits($outfile,$nax,3,\@out_array);
write_crval_cube($outfile,\@tr);
print "DONE\n";
exit;


