#!/usr/bin/perl
#
#
# This program find peaks in a 2D fiber based spectral image
#
#

use Statistics::OLS;
use Math::FFT;
use Math::Stat;
use Math::Spline qw(spline linsearch binsearch);
use Math::Derivative qw(Derivative2);

use Math::Approx;


use Astro::FITS::CFITSIO qw( :longnames :constants );
use PDL;

use PDL::Fit::Polynomial; 
use PDL::Filter::Linear;
use PGPLOT;  # Load PGPLOT module
use PDL::Fit::Gaussian;
use PDL::Image2D;


#require "/work1/ssa/perl/MY/my.pl";
require "/home/sanchez/sda2/code/R3D/my.pl";


if ($#ARGV<9) {
    print "USE: straight_light.pl RAW.fits Spectral_axis[0/1] TRACE.fits width Npoly staight_light.fits clean.fits NY_min NY_max plot\n";
    exit;
}

$infile=$ARGV[0];
$spec_axis=$ARGV[1];
$trace_file=$ARGV[2];
$fwhm=$ARGV[3];
$npoly=$ARGV[4];
$out_file=$ARGV[5];
$clean_file=$ARGV[6];
$ny_min=$ARGV[7];
$ny_max=$ARGV[8];
$plot=$ARGV[9];
print "Reading file $infile ";
$nax=read_naxes($infile);   
@naxis=@$nax;
@tmp_array=read_img($infile);
print "Done\n";
if ($spec_axis==0) {
    @in_array=@tmp_array;
    $nx=$naxis[0];
    $ny=$naxis[1];
} else {
    for ($j=0;$j<$naxis[1];$j++) {
	for ($i=0;$i<$naxis[0];$i++) {
	    $in_array[$i][$j]=$tmp_array[$j][$i];
	}
    }
    $nx=$naxis[1];
    $ny=$naxis[0];
}
print "DONE\n";
print "Reading file $trace_file ";
$nax_tr=read_naxes($trace_file);   
@naxis_tr=@$nax_tr;
@peak_y_max_new=read_img($trace_file);
print "Done\n";
$nx_tr=$naxis_tr[0];
$ny_tr=$naxis_tr[1];

if ($nx_tr!=$nx) {
    print "The X-dimension of the trace file ($trace_file), $nx_tr\n";
    print "does not match that of the original image, $nx\n";
    exit;
}

print "Determining the Straight-light...\n";
for ($i=0;$i<$nx;$i++) {
    my @mask;
    for ($j=0;$j<$ny;$j++) {	
	$mask[$j]=1;
    }    
    for ($j=0;$j<$ny_tr;$j++) {
	$j_cen=int($peak_y_max_new[$j][$i]);
	$j_min=$j_cen-$fwhm;
	$j_max=$j_cen+$fwhm;
	for ($jj=$j_min;$jj<$j_max;$jj++) {
	    $mask[$jj]=0;
	}
    }
    $k=0;
    my @ax;
    my @ay;
    $min=1e12;
    $max=-1e12;
    for ($j=$ny_min;$j<$ny_max;$j++) {	
	if ($mask[$j]==1) {
	    $ax[$k]=$j;
	    $ay[$k]=$in_array[$j][$i];
	    if ($min>$ay[$k]) {
		$min=$ay[$k];
	    }
	    if ($max<$ay[$k]) {
		$max=$ay[$k];
	    }
	    $k++;
	}
    }
#
# We fit a polynomial function
#
    $ax_pdl = pdl(@ax);
    $ay_pdl = pdl(@ay);
    ($s_y,$coeff) = fitpoly1d $ax_pdl,$ay_pdl,$npoly;
    for ($j=0;$j<$npoly;$j++) {
	$c[$j]=$coeff->slice($j)->sclr;
    }
    for ($j=0;$j<$ny;$j++) {
	$sl_out[$j][$i]=0;
	for ($kk=0;$kk<$npoly;$kk++) {
	    $sl_out[$j][$i]=$sl_out[$j][$i]+$c[$kk]*($j**$kk);
	}
    }


    if ($plot==1) {
	pgbegin(0,"/xs",1,1);
	pgsfs(1.2);
	pgscf(2);             # Set character font
	pgslw(2);             # Set line width
	pgsch(1.2);           # Set character height
	pgenv(0,$ny,$min,$max,0,0);
	pglabel("Y-axis","Counts","$i/$nx");
	pgpoint($k,\@ax,\@ay,3);
	for ($j=0;$j<$ny;$j++) {
	    $x=$j;
	    $y=$sl_out[$j][$i];
	    pgsci(2);
	    pgpoint(1,[$x],[$y],1);
	    pgsci(1);
	}


	print "Press Enter"; <stdin>;	
    }
    

}
#print "COSA?";
print "DONE\n";
print "Smoothing...\n";
#print "Smoothing the initial values...\n";
$sum=0;
for ($j=0;$j<11;$j++) {
    for ($i=0;$i<11;$i++) {
	$r=sqrt(($i-4)**2+($j-4)**2);
	$kernel[$i][$j]=exp(-0.5*($r/3)**2);
	$sum=$sum+$kernel[$i][$j];
#	print "$i $j $kernel[$i][$j]\n";
    }
}
$pdl_kernel=pdl(@kernel);
$pdl_kernel=$pdl_kernel/$sum;
$pdl_out=pdl(@sl_out);
@dims=$pdl_out->dims;
#print "*=@dims\n";
$smoothed=med2df($pdl_out,31,1,{Boundary => Reflect});
#$smoothed=med2df($pdl_fwhm2,21,21,{Boundary => Reflect});
#$smoothed2=conv2d($smoothed,$pdl_kernel,{Boundary => Reflect});
#$smoothed3=med2df($smoothed2,5,5,{Boundary => Reflect});
for ($j=0;$j<$ny;$j++) {	
    for ($i=0;$i<$nx;$i++) {
	$msl_out[$j][$i]=$smoothed->at($i,$j);
#	$msl_out[$j][$i]=$sl_out[$j][$i];
	$in_array[$j][$i]=$in_array[$j][$i]-$msl_out[$j][$i];
    }
}

#@msl_out=mean_smooth(\@sl_out,$nx,$ny,2,5);
print "DONE\n";
print "Writting the straight-light solution $out_file\n";
system("rm $out_file");
write_fits($out_file,[$nx,$ny],2,\@msl_out);
print "DONE\n";
print "Writting the cleaned data $clean_file\n";
system("rm $clean_file");
write_fits($clean_file,[$nx,$ny],2,\@in_array);
print "DONE\n";
exit;
